/*
 * @(#)anAlbum.java	1.00 14th Feb 1999
 * 
 * Modification Log:
 * 14th Feb 1999   1.00  Tanmay   Original Version.
 * 10th Jun 1999   1.10  Tanmay   Support for background image. 
 *                                Displays the image number along with caption.
 *                                Displays status while loading images.
 * 09th Oct 1999   1.11  Tanmay   Fixed a bug where the background image was 
 *                                always required to be specified.
 *                                Added check to prevent empty color and image 
 *                                names from giving exception.
 *
 */


import java.applet.Applet;
import java.util.Vector;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;


/**
 * anAlbum 
 * Photo album applet. 
 * This can be used to show off your pictures on your homepage!<br>
 * Has slideshow option and the viewer has next, previous, skip buttons also.<br>
 * Very lightweight. No frills. Just does its job. I like it!<br>
 * <br><br>
 * <u>Applet Parameters:</u><br>
 * SLIDESHOW: If the value is set to anything, slideshow is started by default.<br>
 * FONT: The font to use for buttons and descriptions. In standard Java format. Example: Dialog-bold-12.<br>
 * BGIMAGE: The background image to match your page background.<br>
 * BGCOLOR: The background color to match your page background. In standard Java format. Example: #ffffff. Default is light gray.<br>
 * FGCOLOR: The foreground color to match your page background. In standard Java format. Example: #ffffff. Default is black.<br>
 * DELAY_SECS: Delay beween slideshows in seconds.<br>
 * IMAGE0: The URL of the first image.<br>
 * DESC0: The description for the first image. Default is empty string<br>
 * <br>
 * Similarly IMAGE1, DESC1, IMAGE2, DESC2. Descriptions are not mandatory. 
 * Image and description numbers must be continuous.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.10, 10th Jun, 1999
 */

public class anAlbum extends java.applet.Applet implements java.lang.Runnable, java.awt.event.ActionListener
{
    private Vector              imageVect, descVect;
    private boolean             bSlideShow;
    private long                lDelay;
    private MediaTracker        tracker;
    private int                 iCurrImage = 0;
    private int                 iLastSuccessfulImage = -1;
    private boolean             bContinue;
    private Thread              showThread;
    private Color               backgrColor, foregrColor;
    private Font                textFont;
    private Image               imgBackgr;
    private Image               offScrBackgr;

    // components
    private Button              flipNext, flipPrev, flipStart, flipEnd, slideStartStop;

    /**
     * Read applet parameters and store them.
     * Get all image URLs from the parameters and add the images to a media tracker.
     * Create all components and layout the screen.
     */
    public void init()
    {
        String  sDelay, sBgCol, sFgCol, sFont, sBgImg;
        int     iIndex, iVectIndex;
        
        // get the background image
        if ( (null != (sBgImg = getParameter("BGIMAGE"))) && (sBgImg.length() > 0) )
        {
            URL oneURL;
            try
            {
                if (sBgImg.substring(0, 4).equalsIgnoreCase("http"))
                {
                    oneURL = new URL(sBgImg);
                }
                else
                {
                    oneURL = new URL(getDocumentBase(), sBgImg);
                }
                imgBackgr = getImage(oneURL);
            }
            catch(MalformedURLException me)
            {
                System.err.println("Could not load " + sBgImg);
            }
        }

        // get the background color
        if ( (null != (sBgCol = getParameter("BGCOLOR"))) && (sBgCol.length() > 0) )
        {
            backgrColor = Color.decode(sBgCol);
        }
        if (null == backgrColor)
        {
            backgrColor = Color.decode("#cccccc");
        }
        setBackground(backgrColor);

        // get the foreground color
        if ( (null != (sFgCol = getParameter("FGCOLOR"))) && (sFgCol.length() > 0) )
        {
            foregrColor = Color.decode(sFgCol);
        }
        if (null == foregrColor)
        {
            foregrColor = Color.black;
        }
        setForeground(foregrColor);
        
        // get the font
        if (null != (sFont = getParameter("FONT")))
        {
            textFont = Font.decode(sFont);
        }
        if (null != textFont)
        {
            setFont(textFont);
        }
        
        // get slideshow param
        bSlideShow = (null != getParameter("SLIDESHOW"));
        
        // get the delay time
        if (null != (sDelay = getParameter("DELAY_SECS")))
        {
            lDelay = Long.parseLong(sDelay) * 1000;
        }
        if (0 == lDelay)
        {
            lDelay = 5000; // 5 secs default delay
        }
        
        // get all image params and set them in the vector
        tracker = new MediaTracker(this);
        imageVect = new Vector();
        descVect = new Vector();
        
        for (iIndex=iVectIndex=0; ;iIndex++, iVectIndex++)
        {
            String sImage = "IMAGE"+iIndex;
            String sDescName = "DESC"+iIndex;
            String sURL = getParameter(sImage);
            String sDesc = getParameter(sDescName);
            URL oneURL;
            if (null == sURL)
            {
                break;
            }            
            if (null == sDesc)
            {
                sDesc = "";
            }
            try
            {
                if (sURL.substring(0, 4).equalsIgnoreCase("http"))
                {
                    oneURL = new URL(sURL);
                }
                else
                {
                    oneURL = new URL(getDocumentBase(), sURL);
                }
                Image oneImage = getImage(oneURL);
                tracker.addImage(oneImage, iVectIndex);
                imageVect.addElement(oneImage);
                descVect.addElement(sDesc);
            }
            catch(MalformedURLException mue)
            {
                System.err.println("Could not load " + sURL);
                iVectIndex--;
            }
        }

        // set the canvas, prev button, next button, to start button, to end button
        flipNext        = new Button(">");
        flipPrev        = new Button("<");
        flipStart       = new Button("<<");
        flipEnd         = new Button(">>");
        slideStartStop  = new Button();
        setSlideshowLabel();

        flipNext.addActionListener(this);
        flipPrev.addActionListener(this);
        flipStart.addActionListener(this);
        flipEnd.addActionListener(this);
        slideStartStop.addActionListener(this);

        add(flipStart);
        add(flipPrev);
        add(flipNext);
        add(flipEnd);
        add(slideStartStop);

        if (null != imgBackgr)
        {
            tracker.addImage(imgBackgr, imageVect.size());
        }
        try
        {
            tracker.waitForID(imageVect.size());
        }
        catch(InterruptedException ie)
        {
            // ignore
        }
    }

    /**
     * If slide show to be done, create the animator thread and start it.
     */
    public void start()
    {
        if (bSlideShow) // if slideshow
        {
            startSlideShow();
        }
    }

    /**
     * If slide show is enabled, signal the animator thread to stop.
     */
    public void stop() 
    {
        stopSlideShow();
    }
    
    
    /**
     * If slide show is enabled, signal the animator thread to stop.
     */
    public void destroy()
    {
        stopSlideShow();
    }

    /**
     * Start the slideshow.
     */
    private void startSlideShow()
    {
        if (!bContinue)
        {
            bContinue = true;
            showThread = new Thread(this);  // create the thread
            showThread.start(); // start the thread
            setSlideshowLabel();
        }
    }

    /**
     * Stop the slideshow on its next cycle.
     */
    private void stopSlideShow()
    {
        if (bContinue)
        {
            bContinue = false; // stop the thread on its next cycle
            setSlideshowLabel();
        }
    }

    /**
     * Start slideshow if stopped and stop if running.
     */
    private void toggleSlideShow()
    {
        if (bContinue)  stopSlideShow();
        else            startSlideShow();
    }
    
    /**
     * Check the slide show option and set the label accordingly.
     */
    private void setSlideshowLabel()
    {
        slideStartStop.setLabel(bContinue ? "Stop Slideshow" : "Start Slideshow");
        slideStartStop.validate();
    }

    /**
     * Just call the paint method.
     */
    public void update(Graphics gGraphics)
    {
        paint(gGraphics);
    }

    /**
     * Sets the description for the current image.
     * Paints the current image centered.
     */
    public void paint(Graphics gGraphics)
    {
        Rectangle   appBounds = getBounds();

        // clear the area
        if (null == imgBackgr)
        {
            gGraphics.clearRect(0, 0, appBounds.width, appBounds.height);
        }
        else
        {
            if (null == offScrBackgr)
            {
                int iWidth = imgBackgr.getWidth(this);
                int iHeight = imgBackgr.getHeight(this);
                try
                {
                    offScrBackgr = createImage(appBounds.width, appBounds.height);
                    int iNumHorz = appBounds.width/iWidth;
                    int iNumVert = appBounds.height/iHeight;
                    int iHorzIndex, iVertIndex;
                    Graphics offScrGraphics = offScrBackgr.getGraphics();
                    
                    for (iHorzIndex=0; iHorzIndex <= iNumHorz; iHorzIndex++)
                    {
                        offScrGraphics.drawImage(imgBackgr, iHorzIndex*iWidth, 0, this);
                    }
                    for (iVertIndex=1; iVertIndex <= iNumVert; iVertIndex++)
                    {
                        offScrGraphics.translate(0, iHeight);
                        offScrGraphics.drawImage(offScrBackgr, 0, 0, this);
                    }
                    
                }
                catch(IllegalArgumentException ile)
                {
                    offScrBackgr = null;
                }
            }
            if (null != offScrBackgr)
            {
                gGraphics.drawImage(offScrBackgr, 0, 0, this);
            }
        }

        boolean bSuccess = paintImageAt(gGraphics, iCurrImage);
        if (false == bSuccess)
        {
            if (iLastSuccessfulImage >= 0)
            {
                paintImageAt(gGraphics, iLastSuccessfulImage);
            }
            else
            {
                int             iStartX, iStartY, iElementWidth;
                FontMetrics     fm;
                String          sDesc = "Loading " + imageVect.size() + " images...";
                
                // find out the starting height
                Rectangle   lastBounds = slideStartStop.getBounds();
            
                fm = gGraphics.getFontMetrics();
                iStartY = lastBounds.y + lastBounds.height + fm.getHeight();
                iElementWidth = fm.stringWidth(sDesc);
                if (iElementWidth < appBounds.width)
                {
                    iStartX = (appBounds.width - iElementWidth)/2;
                }
                else
                {
                    iStartX = 0;
                }
            
                // paint the new desc
                gGraphics.drawString(sDesc, iStartX, iStartY);
            }
        }
        else
        {
            iLastSuccessfulImage = iCurrImage;
        }
    }
    

    /**
     * Sets the description for the specified image.
     * Paints the specified image centered.
     */
    public boolean paintImageAt(Graphics gGraphics, int iCurrImage)
    {
        if ((tracker.statusID(iCurrImage, true) & MediaTracker.COMPLETE) != 0) 
        {   // paint the current image
            String          sDesc = (String)descVect.elementAt(iCurrImage);
            int             iStartX, iStartY, iElementWidth;
            FontMetrics     fm;
            
            sDesc = "[" + (iCurrImage+1) + " of " + imageVect.size() + "] " + sDesc;
            
            // find out the starting height
            Rectangle   lastBounds = slideStartStop.getBounds();
            Rectangle   appBounds = getBounds();
            Image       thisImage = (Image)imageVect.elementAt(iCurrImage);
            
            fm = gGraphics.getFontMetrics();
            iStartY = lastBounds.y + lastBounds.height + fm.getHeight();
            iElementWidth = fm.stringWidth(sDesc);
            if (iElementWidth < appBounds.width)
            {
                iStartX = (appBounds.width - iElementWidth)/2;
            }
            else
            {
                iStartX = 0;
            }
            
            // paint the new desc
            gGraphics.drawString(sDesc, iStartX, iStartY);

            iStartY += 5;
            iElementWidth = thisImage.getWidth(this);
            if (iElementWidth < appBounds.width)
            {
                // center the image
                iStartX = (appBounds.width - iElementWidth) / 2;
            }
            else
            {
                iStartX = 0;
            }
            
            // paint the new image
            gGraphics.drawImage(thisImage, iStartX, iStartY, this);
            return true;
        }
        return false;
    }
    

    /**
     * Paint the current image, sleep for delay seconds and advance to the next image.
     * Do this till not signalled to stop.
     */
    public void run()
    {
        if (imageVect.size()==0)
        {
            return; // nothing to show!
        }
        
        while(bContinue)
        {            
            // find the next loaded picture
            nextLoaded();

            // showCurrImage
            repaint();

            // sleep for delay time
            try
            {
                Thread.currentThread().sleep(lDelay);
            }
            catch(InterruptedException ie)
            {
                break;
            }
        }
    }

    /**
     * Handle the button clicks
     */
    public void actionPerformed(ActionEvent ae)
    {
        Component c = (Component)ae.getSource();

        if (c == slideStartStop)
        {
            toggleSlideShow();
        }
        else if (c == flipNext)
        {
            nextLoaded();
        }
        else if (c == flipPrev)
        {
            prevLoaded();
        }
        else if (c == flipStart)
        {
            iCurrImage = 0;
        }
        else if (c == flipEnd)
        {
            iCurrImage = imageVect.size()-1;
        }
        repaint();
        doLayout();
    }
    
    /**
     * Go to the previous loaded picture
     */
    private void prevLoaded()
    {
        int iStatus, iTryImage;
        
        if ((iCurrImage-1) < 0)
        {
            iCurrImage=imageVect.size()-1;
        }
        else
        {
            iCurrImage--;
        }
        
        // find the prev loaded picture
        for(iTryImage=iCurrImage; ; iTryImage=((iTryImage-1) < 0)?(imageVect.size()-1):(iTryImage-1))
        {
            iStatus=tracker.statusID(iTryImage, true);
            if ((iStatus & MediaTracker.COMPLETE) != 0)
            {
                iCurrImage = iTryImage;
                break;
            }
            try
            {
                Thread.currentThread().sleep(200);
            }
            catch(InterruptedException ie)
            {
                // do nothing
            }
        }
    }
    
    /**
     * Go to the next loaded picture
     */
    private void nextLoaded()
    {
        int iStatus, iTryImage;
        
        if ((iCurrImage+1) >= imageVect.size())
        {
            iCurrImage=0;
        }
        else
        {
            iCurrImage++;
        }
        
        // find the next loaded picture
        for(iTryImage=iCurrImage; ; iTryImage=((iTryImage+1) >= imageVect.size())?0:(iTryImage+1))
        {
            iStatus=tracker.statusID(iTryImage, true);
            if ((iStatus & MediaTracker.COMPLETE) != 0)
            {
                iCurrImage = iTryImage;
                break;
            }
            try
            {
                Thread.currentThread().sleep(200);
            }
            catch(InterruptedException ie)
            {
                // do nothing
            }
        }
    }

    /**
     * Gives out parameter info if your browser supports it.
     * JDK Appletviewer does.
     */
    public String[][] getParameterInfo()
    {
        String saAnAppletInfo[][] =
        {
            {"SLIDESHOW",           "string",           "If the value is set to anything, slideshow is started by default"},
            {"FONT",                "string",           "The font to use for buttons and descriptions. In standard Java format. Example: Dialog-bold-12"},
            {"BGIMAGE",             "string",           "The background image to match your page background."},
            {"BGCOLOR",             "string",           "The background color to match your page background. In standard Java format. Example: #ffffff. Default is light gray."},
            {"FGCOLOR",             "string",           "The foreground color to match your page background. In standard Java format. Example: #ffffff. Default is black"},
            {"DELAY_SECS",          "integer",          "Delay beween slideshows in seconds"},            
            {"IMAGE0",              "string",           "The URL of the first image"},
            {"DESC0",               "string",           "The description for the first image. Default is empty string"},
        };

        return saAnAppletInfo;
    }
}
